// Unreal Fighting Framework by Kevin Poretti

#include "FFPlayerController.h"

// FF includes
#include "FFInputBufferComponent.h"

// UE includes
#include "EnhancedInputSubsystems.h"
#include "Kismet/KismetMathLibrary.h"

AFFPlayerController::AFFPlayerController()
{
    InputBuffer = CreateDefaultSubobject<UFFInputBufferComponent>(TEXT("InputBuffer"));
}


void AFFPlayerController::SendInputsToRemote() const
{
}


void AFFPlayerController::ModifyRawInput()
{
    ModifiedInput = RawInput;
}


bool AFFPlayerController::HasMoveInput()
{
    // TODO: configurable deadzone
    return !ModifiedInput.MoveAxes.IsNearlyZero();
}


void AFFPlayerController::ConsumeMoveInput()
{
    RawInput.MoveAxes = FVector2D::ZeroVector;
}


FVector AFFPlayerController::GetInputAsWorldDirection() const
{
	FRotator ViewRotationYaw = FRotator(0.0f, ModifiedInput.LookRot.Yaw, 0.0f);
	FVector ForwardInput = UKismetMathLibrary::GetForwardVector(ViewRotationYaw) * ModifiedInput.MoveAxes.Y;
	FVector RightInput = UKismetMathLibrary::GetRightVector(ViewRotationYaw) * ModifiedInput.MoveAxes.X;

	return ForwardInput + RightInput;
}


void AFFPlayerController::AddCurrentInputToBuffer()
{
    //UnacknowledgedInputs.Push(RawInput);
    InputBuffer->AddInput(ModifiedInput);
    //SendInputsToRemote();
}


bool AFFPlayerController::CheckInputSequence(const FFFInputSequence& InInputSequence)
{
    return InputBuffer->CheckInputSequence(InInputSequence);
}


bool AFFPlayerController::CheckInputSequences(const TArray<FFFInputSequence>& InputSequences)
{
    // no input conditions to check
    if(InputSequences.IsEmpty())
    {
        return true;
    }

    for(const FFFInputSequence& ThisInputSequence : InputSequences)
    {
        if(InputBuffer->CheckInputSequence(ThisInputSequence))
        {
            return true;
        }
    }
    return false;
}


void AFFPlayerController::DisableMostRecentInput()
{
    InputBuffer->DisableMostRecentInput();
}


void AFFPlayerController::SetupInputComponent()
{
    Super::SetupInputComponent();

    if (ULocalPlayer* LocalPlayer = Cast<ULocalPlayer>(Player))
    {
        if (UEnhancedInputLocalPlayerSubsystem* InputSystem = LocalPlayer->GetSubsystem<UEnhancedInputLocalPlayerSubsystem>())
        {
            if (!DefaultInputMapping.IsNull())
            {
                InputSystem->AddMappingContext(DefaultInputMapping.LoadSynchronous(), 0);
            }
        }
    }
}


void AFFPlayerController::FlushPressedKeys()
{
    Super::FlushPressedKeys();

    RawInput.Buttons = 0;
    RawInput.MoveAxes = FVector2D::ZeroVector;
}
